\name{RVineSim}          
\alias{RVineSim}                     

\title{Simulation from an R-vine copula model}

\description{
This function simulates from a given R-vine copula model.
}

\usage{
RVineSim(N, RVM)
}

\arguments{
  \item{N}{Number of d-dimensional observations to simulate.}
  \item{RVM}{An \code{\link{RVineMatrix}} object containing the information of the R-vine copula model.}
}

\value{
An \code{N} x d matrix of data simulated from the given R-vine copula model.
}

\references{
Dissmann, J. F., E. C. Brechmann, C. Czado, and D. Kurowicka (2011).
Selecting and estimating regular vine copulae and application to financial returns.
Submitted for publication.
\url{http://mediatum.ub.tum.de/node?id=1079277}
}

\author{Jeffrey Dissmann}

\seealso{\code{\link{RVineMatrix}}, \code{\link{BiCopSim}}}

\examples{
# define 5-dimensional R-vine tree structure matrix
Matrix = c(5,2,3,1,4,0,2,3,4,1,0,0,3,4,1,0,0,0,4,1,0,0,0,0,1)
Matrix = matrix(Matrix,5,5)

# define R-vine pair-copula family matrix
family = c(0,1,3,4,4,0,0,3,4,1,0,0,0,4,1,0,0,0,0,3,0,0,0,0,0)
family = matrix(family,5,5)

# define R-vine pair-copula parameter matrix
par = c(0,0.2,0.9,1.5,3.9,0,0,1.1,1.6,0.9,0,0,0,1.9,0.5,
        0,0,0,0,4.8,0,0,0,0,0)
par = matrix(par,5,5)

# define second R-vine pair-copula parameter matrix
par2 = matrix(0,5,5)

# define RVineMatrix object
RVM = RVineMatrix(Matrix=Matrix,family=family,par=par,par2=par2,
                  names=c("V1","V2","V3","V4","V5"))

# simulate a sample of size 300 from the R-vine copula model
simdata = RVineSim(300,RVM)
}

